import importlib
import pytest
import sys

modules = [
"Database",
"Database.chemistry",
"Database.chemistry.constants",
"Database.chemistry.reactions",
"Database.chemistry.reactions.StoichiometricReaction",
"Database.chemistry.specie",
"Database.chemistry.specie.Mixtures",
"Database.chemistry.specie.Molecules",
"Database.chemistry.specie.periodicTable",
"Database.chemistry.thermo",
"Database.chemistry.thermo.EquationOfState",
"Database.chemistry.thermo.Thermo",
"Database.chemistry.thermo.Thermo.constantCp",
"Database.chemistry.thermo.Thermo.janaf7",
"Database.data",
"Database._DatabaseClass",
"GLOBALS",
"src",
"src.base",
"src.base.BaseClass",
"src.base.dataStructures",
"src.base.dataStructures.Dictionary",
"src.base.dataStructures.EngineData",
"src.base.dataStructures.EngineData.EngineData",
"src.base.dataStructures.Tabulation",
"src.base.dataStructures.Tabulation.OFTabulation",
"src.base.dataStructures.Tabulation.Tabulation",
"src.base.Filter",
"src.base.Filter.Filter",
"src.base.Filter.LowPassAndResample",
"src.base.Filter.LowPass",
"src.base.Filter.Resample",
"src.base.Filter.UserDefinedFilter",
"src.base.Functions",
"src.base.Functions.functionsForOF",
"src.base.Functions.runtimeWarning",
"src.base.Functions.typeChecking",
"src.base.Logging",
"src.base.Logging.logging_configs",
"src.base.Logging.Logging",
"src.base.Utilities",
"src.engineModel",
"src.engineModel.EngineGeometry",
"src.engineModel.EngineGeometry.ConRod",
"src.engineModel.EngineGeometry.EngineGeometry",
"src.engineModel.EngineModel",
"src.engineModel.EngineModel.EngineModel",
"src.engineModel.EngineModel.SparkIgnitionEngine",
"src.engineModel.EngineTime",
"src.engineModel.EngineTime.EngineTime",
"src.engineModel.EngineTime.SparkIgnitionTime",
"src.engineModel.functionObjects",
"src.engineModel.functions",
"src.engineModel.HeatTransferModel",
"src.engineModel.HeatTransferModel.HeatTransferModel",
"src.engineModel.HeatTransferModel.Woschni",
"src.thermophysicalModels",
"src.thermophysicalModels.laminarFlameSpeedModels",
"src.thermophysicalModels.laminarFlameSpeedModels.LaminarFlameSpeedModel",
"src.thermophysicalModels.laminarFlameSpeedModels.TabulatedLFS",
"src.thermophysicalModels.specie",
"src.thermophysicalModels.specie.reactions",
"src.thermophysicalModels.specie.reactions.functions",
"src.thermophysicalModels.specie.reactions.Reaction",
"src.thermophysicalModels.specie.reactions.Reaction.Reaction",
"src.thermophysicalModels.specie.reactions.Reaction.StoichiometricReaction",
"src.thermophysicalModels.specie.reactions.ReactionModel",
"src.thermophysicalModels.specie.reactions.ReactionModel.DissociationModel",
"src.thermophysicalModels.specie.reactions.ReactionModel.DissociationModel.ConstantDissociationFraction",
"src.thermophysicalModels.specie.reactions.ReactionModel.DissociationModel.DissociationModel",
"src.thermophysicalModels.specie.reactions.ReactionModel.Equilibrium",
"src.thermophysicalModels.specie.reactions.ReactionModel.Inhert",
"src.thermophysicalModels.specie.reactions.ReactionModel.ReactionModel",
"src.thermophysicalModels.specie.reactions.ReactionModel.Stoichiometry",
"src.thermophysicalModels.specie.specie",
"src.thermophysicalModels.specie.thermo",
"src.thermophysicalModels.specie.thermo.EquationOfState",
"src.thermophysicalModels.specie.thermo.EquationOfState.EquationOfState",
"src.thermophysicalModels.specie.thermo.EquationOfState.PerfectGas",
"src.thermophysicalModels.specie.thermo.Thermo",
"src.thermophysicalModels.specie.thermo.Thermo.constantCp",
"src.thermophysicalModels.specie.thermo.Thermo.janaf7",
"src.thermophysicalModels.specie.thermo.Thermo.Thermo",
"src.thermophysicalModels.thermoModels",
"src.thermophysicalModels.thermoModels.CombustionModel",
"src.thermophysicalModels.thermoModels.CombustionModel.CombustionModel",
"src.thermophysicalModels.thermoModels.CombustionModel.NoCombustion",
"src.thermophysicalModels.thermoModels.CombustionModel.PremixedCombustion",
"src.thermophysicalModels.thermoModels.EgrModel",
"src.thermophysicalModels.thermoModels.EgrModel.EgrModel",
"src.thermophysicalModels.thermoModels.EgrModel.FixedCompositionExternalEGR",
"src.thermophysicalModels.thermoModels.EgrModel.StoichiometricMixtureEGR",
"src.thermophysicalModels.thermoModels.StateInitializer",
"src.thermophysicalModels.thermoModels.StateInitializer.mpV",
"src.thermophysicalModels.thermoModels.StateInitializer.StateInitializer",
"src.thermophysicalModels.thermoModels.thermoMixture",
"src.thermophysicalModels.thermoModels.thermoMixture.mixingRules",
"src.thermophysicalModels.thermoModels.thermoMixture.mixingRules.EquationOfState",
"src.thermophysicalModels.thermoModels.thermoMixture.mixingRules.EquationOfState.EquationOfStateMixing",
"src.thermophysicalModels.thermoModels.thermoMixture.mixingRules.EquationOfState.PerfectGasMixing",
"src.thermophysicalModels.thermoModels.thermoMixture.mixingRules.Thermo",
"src.thermophysicalModels.thermoModels.thermoMixture.mixingRules.Thermo.constantCpMixing",
"src.thermophysicalModels.thermoModels.thermoMixture.mixingRules.Thermo.janaf7Mixing",
"src.thermophysicalModels.thermoModels.thermoMixture.mixingRules.Thermo.ThermoMixing",
"src.thermophysicalModels.thermoModels.thermoMixture.ThermoMixture",
"src.thermophysicalModels.thermoModels.ThermoModel",
"src.thermophysicalModels.thermoModels.ThermoState",]

@pytest.mark.parametrize("module", modules)
def test_import_module(module):
    splitModule = module.rsplit(".", 1)
    if len(splitModule) == 1:
        p, m = "", splitModule[0]
    else:
        p, m = splitModule
    exec(f"from ..{p} import {m}")